Notifier Chat Channel

# License

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
Street, Fifth Floor, Boston, MA 02110-1301 USA.

# Field mapping configuration

## Slack

**Configuration**

To produce recipients from entity fields, provide the following mapping:

```yaml
parameters:
  notifier_chat_channel.field_mapping:
    slack:
      transport: slack
      entity_type: user
      bundle: user
      field_name: field_slack
      prefix:
        non_prefixed_as: user  # 'user'|'channel'|false
        user: '@@' # or false ('@@' equals '@' after escaping).
        channel: '#' # or false
        # Whether to enable prefixed names.
        user_enabled: true
        channel_enabled: true
```

**Define a notification**

Take the recipient and configure `SlackOptions` with the recipient with `SlackRecipientInterface::addRecipientToOptions`.

```php
namespace Drupal\my_module;

use Drupal\notifier_chat_channel\Transport\Slack\Recipient\SlackRecipientInterface;
use Symfony\Component\Notifier\Bridge\Slack\SlackOptions;
use Symfony\Component\Notifier\Message\ChatMessage;
use Symfony\Component\Notifier\Notification\ChatNotificationInterface;
use Symfony\Component\Notifier\Notification\Notification;
use Symfony\Component\Notifier\Recipient\RecipientInterface;

final class MyNotification extends Notification implements ChatNotificationInterface {
  public function asChatMessage(RecipientInterface $recipient, ?string $transport = null): ?ChatMessage {
    $options = (new SlackOptions());
      $recipient instanceof SlackRecipientInterface
        ? $recipient->addRecipientToOptions($options)
        : throw new \LogicException('Unexpected recipient of ' . $recipient::class);
      return new ChatMessage('Test Slack message!!', options: $options);
  }
}
```

**Send the notification**

```php
/** @var \Drupal\notifier\Sender\SenderInterface $sender */
$sender = \Drupal::service(SenderInterface::class);

$recipient = User::load(1);
$notification = (new \Drupal\my_module\MyNotification())
  ->channels(['chat/slack']);

/** @var \Drupal\notifier\Sender\SenderInterface $sender */
$sender->send($recipient, $notification);
```
