<?php

declare(strict_types=1);

namespace Drupal\notifier_chat_channel\Transport\Slack\EventListener;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\StringItem;
use Drupal\notifier\Recipients\Event\Recipients;
use Drupal\notifier_chat_channel\Transport\Slack\Recipient\SlackRecipient;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Produce Slack recipients from configured entity fields.
 */
final readonly class SlackRecipients implements EventSubscriberInterface {

  /**
   * @var iterable<\Drupal\notifier_chat_channel\Transport\Slack\SlackFieldConfiguration>
   */
  private iterable $configurations;

  /**
   * @internal
   */
  public function __construct(
    string $configurations,
  ) {
    // @phpstan-ignore-next-line
    $this->configurations = \unserialize($configurations);
  }

  public function onRecipients(Recipients $event): void {
    $entity = $event->for;

    if (!$entity instanceof FieldableEntityInterface) {
      return;
    }

    foreach ($this->configurations as $configuration) {
      if (!($entity->getEntityTypeId() === $configuration->entityTypeId) || !($entity->bundle() === $configuration->bundle)) {
        continue;
      }

      foreach ($entity->get($configuration->fieldName) as $item) {
        \assert($item instanceof StringItem);
        if ($item->isEmpty() === FALSE) {
          try {
            $recipient = SlackRecipient::fromStringItem($item, $configuration);
            if ($recipient !== NULL) {
              $event->addRecipient($recipient);
            }
          }
          catch (\InvalidArgumentException) {
          }
        }
      }
    }
  }

  public static function getSubscribedEvents(): array {
    return [Recipients::class => 'onRecipients'];
  }

}
