<?php

declare(strict_types=1);

namespace Drupal\notifier_chat_channel\Transport\Slack\Recipient;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\StringItem;
use Drupal\notifier\Recipients\Recipient\EntityRecipientInterface;
use Drupal\notifier_chat_channel\Transport\Slack\SlackFieldConfiguration;

/**
 * @internal
 */
abstract class SlackRecipient implements SlackRecipientInterface, EntityRecipientInterface {

  public EntityInterface $entity;

  /**
   * @throws \InvalidArgumentException
   * @internal
   */
  public static function fromStringItem(StringItem $fieldItem, SlackFieldConfiguration $configuration): ?SlackRecipient {
    $rawName = $fieldItem->getString();

    if ($configuration->userEnabled && FALSE !== $configuration->userPrefix && \str_starts_with($rawName, $configuration->userPrefix)) {
      return SlackUserRecipient::create(\substr($rawName, \strlen($configuration->userPrefix)), $fieldItem->getEntity());
    }

    if ($configuration->channelEnabled && FALSE !== $configuration->channelPrefix && \str_starts_with($rawName, $configuration->channelPrefix)) {
      return SlackChannelRecipient::create(\substr($rawName, \strlen($configuration->channelPrefix)), $fieldItem->getEntity());
    }

    if ($configuration->nonPrefixedAs === 'user' && $configuration->userPrefix !== FALSE && FALSE === \str_starts_with($rawName, $configuration->userPrefix)) {
      return SlackUserRecipient::create($rawName, $fieldItem->getEntity());
    }

    if ($configuration->nonPrefixedAs === 'channel' && $configuration->userPrefix !== FALSE && FALSE === \str_starts_with($rawName, $configuration->userPrefix)) {
      return SlackChannelRecipient::create($rawName, $fieldItem->getEntity());
    }

    // This will happen if user or channel is not allowed and the value
    // has a prefix of [by default] @ or #.
    return NULL;
  }

  public function getEntity(): EntityInterface {
    return $this->entity;
  }

}
