<?php

declare(strict_types=1);

namespace Drupal\notifier_chat_channel\Transport\Slack;

use Drupal\notifier_chat_channel\Transport\BundleFieldConfiguration;

/**
 * @internal
 *
 * @phpstan-type SlackFieldMapping array{
 *   entity_type: non-empty-string,
 *   bundle: non-empty-string,
 *   field_name: non-empty-string,
 *   prefix: array{
 *     non_prefixed_as: 'user'|'channel'|false,
 *     user: non-empty-string|false,
 *     channel: non-empty-string|false,
 *     user_enabled: bool,
 *     channel_enabled: bool,
 *   }
 * }
 */
final class SlackFieldConfiguration extends BundleFieldConfiguration {

  /**
   * @var 'user'|'channel'|false
   */
  public string|false $nonPrefixedAs;

  /**
   * @var non-empty-string|false
   */
  public string|false $userPrefix;

  /**
   * @var non-empty-string|false
   */
  public string|false $channelPrefix;

  /**
   * @var bool
   */
  public bool $userEnabled;

  /**
   * @var bool
   */
  public bool $channelEnabled;

  /**
   * @phpstan-param SlackFieldMapping $map
   */
  public static function fromRaw(array $map): static {
    $configuration = new static(
      $map['entity_type'],
      $map['bundle'],
      $map['field_name'],
    );
    $configuration->nonPrefixedAs = $map['prefix']['non_prefixed_as'];
    $configuration->userPrefix = $map['prefix']['user'] ?? '@';
    $configuration->channelPrefix = $map['prefix']['channel'] ?? '#';
    $configuration->userEnabled = $map['prefix']['user_enabled'];
    $configuration->channelEnabled = $map['prefix']['channel_enabled'];
    return $configuration;
  }

}
