<?php

declare(strict_types=1);

namespace Drupal\Tests\notifier_chat_channel\Kernel;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\notifier_chat_channel\Traits\NotifierChatChannelLoggerTrait;
use Symfony\Component\Notifier\Notification\Notification;
use Symfony\Component\Notifier\NotifierInterface;

/**
 * Tests Notifier Chat Channel.
 */
final class NotifierChatChannelTest extends KernelTestBase {

  use NotifierChatChannelLoggerTrait;

  protected static $modules = [
    'notifier',
    'notifier_chat_channel',
  ];

  /**
   * Tests sending a message.
   */
  public function testIntegration(): void {
    $notification = (new Notification())
      ->subject('Test message!')
      ->channels(['chat/fake_chat']);

    static::notifier()->send($notification);

    $logs = $this->getLogs();
    static::assertCount(1, $logs);
    static::assertEquals('New Chat message without specified recipient!: Test message!', $logs[0]['message']);
  }

  public function register(ContainerBuilder $container): void {
    parent::register($container);
    $container->setParameter('chatter.transports', [
      'fake_chat' => 'fakechat+logger://default',
    ]);
    $this->loggerRegister($container);
  }

  protected function tearDown(): void {
    $this->loggerTeardown();
    parent::tearDown();
  }

  private static function notifier(): NotifierInterface {
    return \Drupal::service(NotifierInterface::class);
  }

}
