<?php

declare(strict_types=1);

namespace Drupal\Tests\notifier_chat_channel\Unit;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\StringItem;
use Drupal\notifier_chat_channel\Transport\Slack\Recipient\SlackChannelRecipient;
use Drupal\notifier_chat_channel\Transport\Slack\Recipient\SlackRecipient;
use Drupal\notifier_chat_channel\Transport\Slack\Recipient\SlackUserRecipient;
use Drupal\notifier_chat_channel\Transport\Slack\SlackFieldConfiguration;
use Drupal\Tests\UnitTestCase;

/**
 * @phpstan-import-type SlackFieldMapping from \Drupal\notifier_chat_channel\Transport\Slack\SlackFieldConfiguration
 */
final class NotifierChatChannelSlackTest extends UnitTestCase {

  /**
   * @covers \Drupal\notifier_chat_channel\Transport\Slack\Recipient\SlackRecipient
   * @phpstan-param SlackFieldMapping $map
   * @phpstan-param class-string $expectedRecipientInstanceOf
   * @dataProvider providerSlackFieldConfigurationRecipients
   */
  public function testSlackFieldConfigurationRecipients(array $map, string $getStringReturns, ?string $expectedRecipientInstanceOf, string $expectedRecipientValue): void {
    $fieldItem = \Mockery::mock(StringItem::class);
    $fieldItem->expects('getString')
      ->andReturn($getStringReturns);
    $fieldItem->expects('getEntity')
      ->andReturn(\Mockery::mock(EntityInterface::class));

    $configuration = SlackFieldConfiguration::fromRaw($map);
    $recipient = SlackRecipient::fromStringItem(
      $fieldItem,
      $configuration,
    );

    if ($expectedRecipientInstanceOf === NULL) {
      static::assertNull($recipient);
      return;
    }

    static::assertInstanceOf($expectedRecipientInstanceOf, $recipient);
    if ($expectedRecipientInstanceOf === SlackUserRecipient::class) {
      static::assertInstanceOf(SlackUserRecipient::class, $recipient);
      static::assertEquals($expectedRecipientValue, $recipient->userName);
    }
    elseif ($expectedRecipientInstanceOf === SlackChannelRecipient::class) {
      static::assertInstanceOf(SlackChannelRecipient::class, $recipient);
      static::assertEquals($expectedRecipientValue, $recipient->channelName);
    }
    else {
      throw new \LogicException($expectedRecipientInstanceOf . ' is not covered.');
    }
  }

  public static function providerSlackFieldConfigurationRecipients(): \Generator {
    yield [
      [
        'entity_type' => '',
        'bundle' => '',
        'field_name' => '',
        'prefix' => [
          'non_prefixed_as' => 'user',
          'user' => '@',
          'channel' => '#',
          'user_enabled' => TRUE,
          'channel_enabled' => TRUE,
        ],
      ],
      '@foo',
      SlackUserRecipient::class,
      'foo',
    ];

    yield [
      [
        'entity_type' => '',
        'bundle' => '',
        'field_name' => '',
        'prefix' => [
          'non_prefixed_as' => 'user',
          'user' => '@',
          'channel' => '#',
          'user_enabled' => TRUE,
          'channel_enabled' => TRUE,
        ],
      ],
      '#foo',
      SlackChannelRecipient::class,
      'foo',
    ];

    yield [
      [
        'entity_type' => '',
        'bundle' => '',
        'field_name' => '',
        'prefix' => [
          'non_prefixed_as' => 'user',
          'user' => '@',
          'channel' => '#',
          'user_enabled' => TRUE,
          'channel_enabled' => TRUE,
        ],
      ],
      'foo',
      SlackUserRecipient::class,
      'foo',
    ];

    yield [
      [
        'entity_type' => '',
        'bundle' => '',
        'field_name' => '',
        'prefix' => [
          'non_prefixed_as' => 'channel',
          'user' => '@',
          'channel' => '#',
          'user_enabled' => TRUE,
          'channel_enabled' => TRUE,
        ],
      ],
      'foo',
      SlackChannelRecipient::class,
      'foo',
    ];

    yield [
      [
        'entity_type' => '',
        'bundle' => '',
        'field_name' => '',
        'prefix' => [
          'non_prefixed_as' => FALSE,
          'user' => '@',
          'channel' => '#',
          'user_enabled' => TRUE,
          'channel_enabled' => TRUE,
        ],
      ],
      'foo',
      NULL,
      '',
    ];
  }

}
